(*
 * SimpleSystems.m
 *
 * Andrew D. Lewis
 * University of Warwick
 * 10 October 1997
 *
 * Does computations for simple mechanical systems with symmetry.  That is,
 * computes things like the mechanical connection, the locked inertia
 * tensor, etc.  You should have your coordinates arranged so the base
 * space coordinates are the first DimQ-DimG coordinates, and the G
 * coordinates are the last DimG coordinates.  The basis you choose for
 * the Lie algebra should be the same as the one given by the coordinate
 * vectors for the G parameterisation at the identity.
 *
 *)

BeginPackage["SimpleSystems`"]

HorizontalBundle::usage = "HorizontalBundle[g,InfGen] computes the
horizontal distribution for a system with Riemannian metric g.  InfGen
is a matrix whose columns are the infinitesimal generators corresponding to
basis elements for Lie algebra."

ConnectionForm::usage = "ConnectionForm[HBun,InfGen] computes the connection
one-form where HBun is the horizontal distribution.  InfGen
is a matrix whose columns are the infinitesimal generators corresponding to
basis elements for Lie algebra.  The result is waiting to eat a vector and
return a Lie algebra element."

LocalConnectionForm::usage = "LocalConnectionForm[Conn,Adg] computes the
local connection form where Conn is the mechanical connection as returned
by \"ConnectionForm\" and Adg is the matrix for the adjoint representation."

CurvatureForm::usage = "CurvatureForm[Conn] computes the curvature form for
the principal connection Conn.  The result is waiting to eat two vectors
and return a Lie algebra element."

LocalCurvatureForm::usage = "LocalCurvatureForm[Conn,LASC,x] computes the local
curvature form with Conn the local connection form, LASC are the Lie algebra
structure constants, and x are the coordinates for the base space."

LockedInertia::usage = "LockedInertia[g,InfGen] computes the locked inertia
tensor for the system with Riemannian metric g.  InfGen
is a matrix whose columns are the infinitesimal generators corresponding to
basis elements for Lie algebra.  The result is waiting to eat two Lie
algebra vectors and return a number."

LocalLockedInertia::usage = "LocalLockedInertia[LIT,Adg] computes the local
locked inertia tensor where LIT is the locked inertia tensor, and Adg is the
matrix for the adjoint representation."

ReducedMetric::usage = "ReducedMetric[g,Connloc,LITloc] computes the
reduced Riemannian metric.  g is the inertia matrix, Connloc is the local
connection form, and LITloc is the local locked inertia tensor."

ReducedEquations::usage =
"ReducedEquations[gred,LIT,Conn,LASC,x,xdot,xddot,xi,xidot] computes the
reduced Euler-Lagrange equations.  gred is the reduced metric, LIT is the
local locked inertia tensor, Conn is the local mechanical connection form,
LASC are the Lie algebra structure constants, and the remainder of the
arguments are variable names."

EPEquations::usage = "EPEquations[LIT,LASC,xi,xidot] computes the
Euler-Poincare equations associated with the locked inertia tensor LIT, the
Lie algebra structure constants LASC, and the variables xi.  xidot are the
variables representing the time derivative of xi."

Begin["Private`"]

Print["\nPackage \"SimpleSystems\" defines: HorizontalBundle, ConnectionForm, LocalConnectionForm, CurvatureForm, LocalCurvatureForm, LockedInertia, LocalLockedInertia, ReducedMetric, ReducedEquations, EPEquations."];

HorizontalBundle[g_, InfGen_] := Module[{},
(*
 * Computes the horizontal distribution.
 *)
  NullSpace[InfGen.g]
  ];

ConnectionForm[HBun_, InfGen_] := Module[{DimQ,DimG,P,Pinv},
(*
 * Computes the connection form for the horizontal distribution HBun.
 *)
  DimG = Length[InfGen];
  DimQ = Length[Transpose[InfGen]];
  P = Table[0,{i,DimQ},{j,DimQ}];
  P = Table[If[i < DimQ-DimG+1,HBun[[i]][[j]],InfGen[[i-DimQ+DimG]][[j]]],
            {i,DimQ},{j,DimQ}];
  Pinv = Inverse[P];
  Transpose[Table[Pinv[[i]][[j]],{i,DimQ},{j,DimQ-DimG+1,DimQ,1}]]
  ];

LocalConnectionForm[Conn_, Adg_] := Module[{DimG,DimQ,Connloc1},
(*
 * Computes the local connection form.
 *)
  DimG = Length[Conn];
  DimQ = Length[Transpose[Conn]];
  Connloc1 = Simplify[Inverse[Adg].Conn];
  Table[Connloc1[[i]][[j]],{i,DimG},{j,DimQ-DimG}]
  ];

CurvatureForm[Conn_] := Module[{},
(*
 * Computes the curvature of Conn.
 *)
  
  ];

LocalCurvatureForm[Conn_, LASC_, x_] := Module[{DimQ,DimG},
(*
 * Computes the local curvature form.
 *)
  DimG = Length[Conn];
  DimQ = DimG+Length[x];
  Table[D[Conn[[i]][[j]],x[[k]]]-D[Conn[[i]][[k]],x[[j]]]-
        Sum[LASC[[i]][[l]][[m]] Conn[[l]][[j]] Conn[[m]][[k]],
        {l,DimG},{m,DimG}],{i,DimG},{j,DimQ-DimG},{k,DimQ-DimG}]
  ];

LockedInertia[g_, InfGen_] := Module[{DimG},
(*
 * Computes the locked inertia tensor.
 *)
  DimG = Length[InfGen];
  Table[InfGen[[i]].g.InfGen[[j]],{i,DimG},{j,DimG}]
  ];

LocalLockedInertia[LIT_, Adg_] := Module[{},
(*
 * Computes the local locked inertia tensor.
 *)
  Transpose[Adg].LIT.Adg
  ];
  
ReducedMetric[g_ ,Connloc_, LIT_] := Module[{DimQ,DimG,K,Extra},
(*
 * Computes the reduced metric.
 *)
  DimG = Length[LIT];
  DimQ = Length[g];
  K = Table[g[[i]][[j]],{i,DimQ-DimG},{j,DimQ-DimG}];
  Extra = Table[Sum[LIT[[k]][[l]] Connloc[[k]][[i]] Connloc[[l]][[j]],
                    {k,DimG},{l,DimG}],
                {i,DimQ-DimG},{j,DimQ-DimG}];
  Table[K[[i]][[j]] - Extra[[i]][[j]],{i,DimQ-DimG},{j,DimQ-DimG}]
  ];

ReducedEquations[gred_,LIT_,Conn_,Curv_,LASC_,x_,xdot_,xddot_,xi_,xidot_] := Module[{DimQ,DimG,LC,xiflat,term1,term2,term3,term4,term5,term6,term7,term8},
(*
 * Computes the reduced Euler-Lagrange equations.
 *)
  DimG = Length[LIT];
  DimQ = DimG+Length[gred];
  LC = LeviCivita[gred,x];
  xiflat = Table[Sum[LIT[[i]][[j]] xi[[j]],{j,DimG}],{i,DimG}];
  term1 = Table[Sum[LC[[i]][[j]][[k]] xdot[[j]] xdot[[k]],
          {j,DimQ-DimG},{k,DimQ-DimG}],{i,DimQ-DimG}];
  term2 = gred.(xddot-term1);
  term3 = (1/2)Table[Sum[D[LIT[[j]][[k]],x[[i]]],{j,DimG},{k,DimG}],
          {i,DimQ-DimG}];
  term4 = Table[Sum[Curv[[i]][[j]][[k]] xdot[[k]] xiflat[[i]],
                {k,DimQ-DimG},{i,DimG}],{j,DimQ-DimG}];
  term5 = Table[Sum[LASC[[i]][[j]][[k]] Conn[[j]][[l]] xi[[k]] xiflat[[i]],
          {i,DimG},{j,DimG},{k,DimG}],{l,DimQ-DimG}];
  term6 = (1/2)Table[Sum[D[LIT[[i]][[j]],x[[k]]] xdot[[k]] xi[[j]],
          {k,DimQ-DimG},{j,DimG}],{i,DimG}];
  term7 = Table[Sum[LASC[[i]][[j]][[k]] Conn[[j]][[l]] xdot[[l]] xiflat[[i]],
          {i,DimG},{j,DimG},{l,DimQ-DimG}],{k,DimG}];
  term8 = EPEquations[LIT,LASC,xi,xidot];
  (*Table[If[i<=DimQ-DimG,
           term2[[i]]+term3[[i]]-term4[[i]]+term5[[i]],
           term8[[i-DimQ+DimG]]+term6[[i-DimQ+DimG]]-term7[[i-DimQ+DimG]]],
       {i,DimQ}];*)
  {term2+term3-term4+term5,term8+term6-term7}
  ];

EPEquations[LIT_, LASC_, xi_, xidot_] :=
Module[{DimG,xiflat,tempone,temptwo},
(*
 * Computes the Euler-Poincare equations.
 *)
  DimG = Length[LIT];
  xiflat = Table[Sum[LIT[[i]][[j]] xi[[j]],{j,DimG}],{i,DimG}];
  tempone = Table[Sum[LIT[[i]][[j]] xidot[[j]],{j,DimG}],{i,DimG}];
  temptwo = Table[Sum[LASC[[j]][[k]][[i]] xi[[k]] xiflat[[j]],
                      {j,DimG},{k,DimG}],{i,DimG}];
  tempone-temptwo
  ];

End[]
EndPackage[]
